import whisper
import torchaudio
import logging
import torch
from pathlib import Path
from typing import List, Dict, Union

def transcribe_segments(
    audio_path: str,
    speech_timestamps: List[Dict[str, float]],
    model_name: str,
    device: str
) -> List[Dict[str, Union[float, str]]]:
    """
    Транскрибирует аудиосегменты, используя OpenAI Whisper.
    
    Args:
        audio_path (str): Путь к полному аудиофайлу.
        speech_timestamps (list): Список словарей с начальным и конечным временем 
                                   каждого сегмента речи.
        model_name (str): Имя модели Whisper (например, 'base').
        device (str): Устройство для выполнения ('cuda' или 'cpu').

    Returns:
        list: Список транскрибированных сегментов.
    """
    model = whisper.load_model(model_name, device=device)
    
    transcriptions = []
    
    audio_path = Path(audio_path)
    if not audio_path.exists():
        raise FileNotFoundError(f"Аудиофайл не найден: {audio_path}")
        
    full_waveform, sample_rate = torchaudio.load(audio_path)
    
    if full_waveform.shape[0] > 1:
        full_waveform = torch.mean(full_waveform, dim=0, keepdim=True)
    if sample_rate != 16000:
        resampler = torchaudio.transforms.Resample(orig_freq=sample_rate, new_freq=16000)
        full_waveform = resampler(full_waveform)
        sample_rate = 16000
    
    for segment in speech_timestamps:
        start_time = segment['start']
        end_time = segment['end']
        
        start_sample = int(start_time * sample_rate)
        end_sample = int(end_time * sample_rate)
        
        audio_segment = full_waveform[:, start_sample:end_sample].squeeze().numpy()
        
        if audio_segment.size == 0:
            transcriptions.append({
                'start': start_time,
                'end': end_time,
                'text': ''
            })
            continue

        try:
            result = model.transcribe(audio_segment, language="ja")
            
            transcriptions.append({
                'start': start_time,
                'end': end_time,
                'text': result['text'].strip()
            })
        except Exception as e:
            logging.exception(f"Ошибка при транскрипции сегмента с помощью Whisper [{start_time:.2f}s - {end_time:.2f}s]: {e}")
            transcriptions.append({
                'start': start_time,
                'end': end_time,
                'text': ''
            })
            
    return transcriptions


import os
import subprocess
import logging

def separate_vocals(audio_path: str, device: str, output_dir: str = "separated"):
    """
    Separates vocal track from an audio file using Demucs.
    
    Args:
        audio_path (str): The path to the input audio file.
        output_dir (str): The directory where the separated tracks will be saved.
    
    Returns:
        str: The path to the separated vocal track.
    """
    logging.info(f"Отделение вокала из {audio_path}...")
    
    # Создаем выходную директорию, если она не существует
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    
    # Команда для Demucs, которая разделяет аудио на 4 дорожки (вокал, барабаны, бас, другое)
    command = [
        "demucs",
        "--device", device, # Добавляем флаг устройства
        audio_path,
        "-o", output_dir
    ]
    
    # Запускаем команду
    subprocess.run(command, check=True)
    
    # Формируем путь к файлу с вокалом
    vocal_track_name = os.path.splitext(os.path.basename(audio_path))[0]
    vocal_track_path = os.path.join(output_dir, "htdemucs", vocal_track_name, "vocals.wav")
    
    if os.path.exists(vocal_track_path):
        logging.info(f"Вокал успешно отделен и сохранен в {vocal_track_path}")
        return vocal_track_path
    else:
        logging.critical("Ошибка: файл с вокалом не найден.")
        return None



import torch
import torchaudio
from omegaconf import OmegaConf  # noqa: F401

# Загрузка модели silero-vad
# Если вы используете локальную модель, замените URI на путь к файлу.
model, utils = torch.hub.load(
    repo_or_dir="snakers4/silero-vad",
    model="silero_vad",
    force_reload=False,
    onnx=False
)
(get_speech_timestamps, save_audio, read_audio, VADIterator, collect_chunks) = utils

def detect_speech_segments(audio_path: str):
    """
    Детектирует сегменты речи в аудиофайле с помощью silero-vad.
    
    Args:
        audio_path (str): Путь к аудиофайлу.
        
    Returns:
        list: Список словарей с начальным и конечным временем каждого 
              сегмента речи.
    """
    # Загрузка и обработка аудиофайла
    waveform, sample_rate = torchaudio.load(audio_path)
    
    # Конвертация в моно, если необходимо
    if waveform.shape[0] > 1:
        waveform = torch.mean(waveform, dim=0, keepdim=True)
    
    # Ресэмплинг до 16000 Гц, если необходимо
    if sample_rate != 16000:
        resampler = torchaudio.transforms.Resample(orig_freq=sample_rate, new_freq=16000)
        waveform = resampler(waveform)
        sample_rate = 16000
    
    # Определение сегментов речи
    speech_timestamps = get_speech_timestamps(
        waveform, 
        model, 
        sampling_rate=sample_rate, 
        return_seconds=True
    )
    
    return speech_timestamps



from typing import List
from punctuators.models import PunctCapSegModelONNX


model: PunctCapSegModelONNX = PunctCapSegModelONNX.from_pretrained(
    "1-800-BAD-CODE/xlm-roberta_punctuation_fullstop_truecase"
)

def add_punctuation_with_xlm(texts: List[str]) -> List[List[str]]:
    """
    Добавляет японскую пунктуацию в список текстов с помощью
    модели xlm-roberta.

    Args:
        texts (List[str]): Список строк без пунктуации.

    Returns:
        List[List[str]]: Список, где каждый элемент — это список
                         предложений с пунктуацией.
    """
    if not texts:
        return []
    
    # Модель обрабатывает список текстов и возвращает список списков предложений
    punctuated_texts = model.infer(texts)
    
    return punctuated_texts



import os
import argparse
import tempfile
import shutil
import torch
import logging
import sys
from typing import List, Dict

from .asr_whisper import transcribe_segments
from .separator import separate_vocals
from .vad_detector import detect_speech_segments
from .punctuator import add_punctuation_with_xlm
from .srt_formatter import segments_to_srt

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

def merge_close_segments(
    timestamps: List[Dict], max_silence_s: float = 0.4
) -> List[Dict]:
    if not timestamps:
        return []
    merged_timestamps = []
    current_segment = timestamps[0].copy()
    for next_segment in timestamps[1:]:
        if (next_segment['start'] - current_segment['end']) < max_silence_s:
            current_segment['end'] = next_segment['end']
        else:
            merged_timestamps.append(current_segment)
            current_segment = next_segment.copy()
    merged_timestamps.append(current_segment)
    return merged_timestamps

def process_audio(input_path: str, output_path: str, model_name: str, device: str):
    """
    Полный конвейер обработки аудиофайла для генерации субтитров с выбором ASR движка.

    Args:
        input_path (str): Путь к исходному аудио/видео файлу.
        output_path (str): Путь для сохранения итогового .srt файла.
        asr_engine (str): Выбранный движок ASR ('whisper' или 'riva').
        model_name (str): Имя модели для выбранного движка.
        device (str): Устройство для выполнения ('cuda' или 'cpu').
    """
    logging.info("--- Запуск процесса создания субтитров ---")

    with tempfile.TemporaryDirectory() as temp_dir:
        logging.info(f"Создана временная директория: {temp_dir}")

        # --- Шаг 1: Отделение вокала ---
        logging.info(f"[1/5] Отделение вокала из файла: {input_path}")
        vocals_path = separate_vocals(input_path, device=device, output_dir=temp_dir)
        if not vocals_path:
            logging.critical("Не удалось отделить вокал. Процесс прерван.")
            return
        logging.info(f"Вокал успешно сохранен в: {vocals_path}")

        # --- Шаг 2: Детекция речи (VAD) ---
        logging.info("[2/5] Обнаружение сегментов речи...")
        speech_timestamps = detect_speech_segments(vocals_path)
        if not speech_timestamps:
            logging.critical("Речь не обнаружена. Процесс прерван.")
            return
        logging.info(f"Обнаружено {len(speech_timestamps)} сегментов речи.")

        speech_timestamps = merge_close_segments(speech_timestamps)
        logging.info(f"После объединения осталось {len(speech_timestamps)} сегментов.")

        # --- Шаг 3: Транскрибация речи (ASR) ---
        logging.info("[3/5] Транскрибация сегментов...")
        transcribed_segments = transcribe_segments(
            audio_path=vocals_path,
            speech_timestamps=speech_timestamps,
            model_name=model_name,
            device=device
        )
        logging.info("Транскрибация завершена.")

        # --- Шаг 4: Добавление пунктуации ---
        logging.info("[4/5] Добавление пунктуации...")
        texts_to_punctuate = [seg['text'] for seg in transcribed_segments if seg['text']]
        if texts_to_punctuate:
            punctuated_lists = add_punctuation_with_xlm(texts_to_punctuate)
            punctuated_texts = [" ".join(sentences) for sentences in punctuated_lists]
            text_iterator = iter(punctuated_texts)
            for segment in transcribed_segments:
                if segment['text']:
                    try:
                        segment['text'] = next(text_iterator)
                    except StopIteration:
                        logging.warning("Предупреждение: количество сегментов с текстом и результатов пунктуации не совпадает.")
                        break
        logging.info("Пунктуация добавлена.")

        # --- Шаг 5: Форматирование в SRT и сохранение ---
        logging.info("[5/5] Форматирование в SRT и сохранение файла...")
        srt_content = segments_to_srt(transcribed_segments)
        try:
            with open(output_path, 'w', encoding='utf-8') as f:
                f.write(srt_content)
            logging.info("\n--- Процесс успешно завершен! ---")
            logging.info(f"Субтитры сохранены в файл: {output_path}")
        except IOError as e:
            logging.critical(f"Ошибка при записи в файл '{output_path}': {e}")
    logging.info("Временные файлы очищены.")


def main():
    """
    Главная функция для запуска из командной строки.
    """
    parser = argparse.ArgumentParser(
        description="Создает субтитры (.srt) из любого видео или аудиофайла с выбором ASR движка.",
        formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        "input_file",
        type=str,
        help="Путь к исходному видео/аудио файлу."
    )
    parser.add_argument(
        "-o", "--output",
        type=str,
        help="Путь к итоговому .srt файлу (по умолчанию: имя_файла.srt)."
    )
    parser.add_argument(
        "-m", "--model",
        type=str,
        default="small",
        help="Имя модели для выбранного движка (по умолчанию: 'small' для Whisper)."
    )
    parser.add_argument(
        "-d", "--device",
        type=str,
        default=None,
        choices=['cpu', 'cuda'],
        help="Устройство для вычислений (cpu или cuda). По умолчанию определяется автоматически."
    )
    args = parser.parse_args()

    if not shutil.which("ffmpeg"):
        logging.critical("Ошибка: FFmpeg не найден. Пожалуйста, установите FFmpeg и убедитесь, что он доступен в системном PATH.")
        return
    
    if not os.path.exists(args.input_file):
        logging.critical(f"Ошибка: Входной файл не найден по пути: {args.input_file}")
        return

    # Выбор устройства
    if args.device is None:
        device = "cuda" if torch.cuda.is_available() else "cpu"
        logging.warning(f"Устройство не указано, используется автоопределение: {device.upper()}")
    else:
        device = args.device
        if device == "cuda" and not torch.cuda.is_available():
            logging.error("Ошибка: Выбрано устройство 'cuda', но оно недоступно. Используется 'cpu'.")
            device = "cpu"

    # Определяем путь для выходного файла
    if args.output:
        output_file_path = args.output
    else:
        base_name = os.path.splitext(os.path.basename(args.input_file))[0]
        output_file_path = f"{base_name}.srt"
        
    process_audio(args.input_file, output_file_path, args.model, device)

if __name__ == '__main__':
    # Добавляем, чтобы main_logic.py работал как пакет
    current_dir = os.path.dirname(os.path.abspath(__file__))
    sys.path.append(current_dir)
    main()


import datetime
from typing import List, Dict

def _format_srt_time(seconds: float) -> str:
    """
    Форматирует время из секунд в формат SRT: ЧЧ:ММ:СС,ммм.
    
    Args:
        seconds (float): Время в секундах.
        
    Returns:
        str: Отформатированная строка времени.
    """
    delta = datetime.timedelta(seconds=seconds)

    hours, remainder = divmod(delta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    milliseconds = delta.microseconds // 1000
    
    return f"{hours:02}:{minutes:02}:{seconds:02},{milliseconds:03}"

def segments_to_srt(segments: List[Dict]) -> str:
    """
    Преобразует список сегментов в одну строку в формате SRT.

    Args:
        segments (List[Dict]): Список словарей, где каждый словарь
                               содержит ключи 'start', 'end' и 'text'.

    Returns:
        str: Содержимое SRT-файла в виде строки.
    """
    srt_blocks = []

    for i, segment in enumerate(segments, start=1):
        # Пропускаем сегменты без текста, чтобы не создавать пустые субтитры
        if not segment.get('text', '').strip():
            continue
            
        start_time = _format_srt_time(segment['start'])
        end_time = _format_srt_time(segment['end'])
        text = segment['text']
        
        block = f"{i}\n{start_time} --> {end_time}\n{text}\n"
        srt_blocks.append(block)
        
    return "\n".join(srt_blocks)